use std::path::PathBuf;

use crate::error::CustomError;

pub(crate) type RefDependencyManager = std::sync::Arc<std::sync::Mutex<DependencyManager>>;

#[derive(Debug)]
pub(crate) struct DependencyManager {
    /// A stack of paths for the files being visited.
    ///
    /// The last entry is the current file being processed. This can be used for handling relative-path links.
    file_stack: Vec<PathBuf>,
}

impl DependencyManager {
    pub(crate) fn new() -> Self {
        DependencyManager {
            file_stack: Vec::new(),
        }
    }

    pub(crate) fn push_file<P>(&mut self, path: P) -> Result<(), CustomError>
    where
        P: Into<PathBuf>,
    {
        self.file_stack.push(path.into());
        Ok(())
    }

    pub(crate) fn pop_file(&mut self) -> Result<(), CustomError> {
        self.file_stack
            .pop()
            .expect("Popped more files off the dependency manager file stack than exist.");
        Ok(())
    }
}
